
#
# The Result class wraps a HTTP Response object and provides some test framework-specific additional result
# tracking.
#
# The ServiceAPI#do_request* methods returns this Result object for each test.
#
class Result

  attr_reader :response
  attr_reader :is_ok
  attr_reader :message
  attr_accessor :allow_retry # if false, do not retry test even if test file requests retries
  attr_accessor :abort_suite_run # if true, framework will end test suite run immediately

  #-------------------------------------------------------------------------------------------------------------------
  # Constructor.
  #
  def initialize(response)
    @response = response
    @is_ok = true
    @allow_retry = true
    @message = ""
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Fatal error factory. A service module should return a fatal_error when it encounters a problem which cannot
  # change by re-trying the same test again. The most common case is if a required parameter is missing.
  #
  def self.fatal_error(message)
    res = Result.new(nil)
    res.error(message)
    res.allow_retry = false
    return res
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Constructs a Result object which tells the framework to abort the current test suite.
  #
  # A service module should generally never return this. Avoid it! Even if a particular misconfiguration is fatal
  # for a certain subset of tests, it is possible the suite may have many other tests which can still run.
  #
  # It is provided for cases where a service module detects a misconfiguration so fatal that there truly is no point
  # in attempting to run any more tests because you know for sure they will all fail.
  #
  def self.abort_suite(message)
    res = Result.new(nil)
    res.error(message)
    res.allow_retry = false
    res.abort_suite_run = true
    return res
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Flags this Result as erroneous.
  # After one call to this method, is_ok will return false.
  # A descriptive message string needs to be provided, explaining why the result is erroneous.
  # This method can be invoked multiple times, if multiple errors are noticed on the response.
  # The message(s) will be available via the message method.
  #
  def error(msg)
    @is_ok = false
    add_message(msg)
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Adds a message sentence to the error message line.
  # Typically callers should invoke error() above.
  #
  def add_message(msg)
    @message = @message + msg
    if (@message[-1,1] != " ")
      @message = @message + " "
    end
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Returns the HTTP response code sent by server (as a String)
  #
  def code
    return (-1) if @response == nil
    return @response.code
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Returns the HTTP response body sent by server.
  #
  def body
    return "" if @response == nil
    return @response.body
  end

end
