
#
# This is the base class for all service module classes.
#

require 'net/http'
require 'json'
require 'Result'

class ServiceAPI

  #-------------------------------------------------------------------------------------------------------------------
  # Returns a string which can be eval'd to set a variable or return an error.
  #
  # A very common pattern in service module methods is to attempt to get a value from state or return
  # a fatal error Result if it is missing. This convenience method allows reducing the code segment:
  #
  #     xyz = state.get('xyz')
  #     if (xyz == nil)
  #       return Result.fatal_error("xyz required")
  #     end
  #
  # to this:
  #
  #     xyz = ""; eval(need :xyz, state)
  #
  # (If only ruby has LISP macros, this could be so much cleaner...)
  #
  def need(symbol, state)
    x = state.get(symbol.to_s)
    if (x == nil)
      return "return Result.fatal_error(\"#{symbol.to_s} required\")"
    end
    return "#{symbol.to_s} = state.get('#{symbol.to_s}')"
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Generate a random UUID.
  #
  # Attempts a few different ways, hopefully one of them work. If all fails, die.
  #
  def uuid
    begin
      require 'securerandom'
      uuid = SecureRandom.uuid()

    rescue Exception => e
      if (File.exist?("/usr/bin/uuidgen")) # Centos e2fsprogs package
        uuid = `/usr/bin/uuidgen`
        return uuid.chomp

      elsif (File.exist?("/usr/bin/uuid")) # Debian uuid package
        uuid = `/usr/bin/uuid`
        return uuid.chomp

      else
        die("Unable to generate UUIDs")
      end
    end
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Convenience method to build a string of HTTP query parameters.
  #
  def add_param(params, param)
    if (param == nil || param.length() == 0)
      return params
    end
    if (params == nil || params.length() == 0)
      return "?#{param}"
    end
    return "#{params}&#{param}"
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Show details about request being sent.
  #
  def show_req(request)
    if ($LOG_LEVEL >= 2)
      puts "---[ Request ]----------------------------------------------------"
      puts "#{request.method} #{request.path}"
      request.each_header { |name,value|
        puts "#{name}: #{value}"
      }
      puts "\n#{request.body}\n"
      puts "------------------------------------------------------------------"
    end
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Show details about response received.
  #
  def show_response(response)
    if ($LOG_LEVEL >= 2)
      puts "---[ Response ]---------------------------------------------------"
      puts "#{response.code} #{response.message}"
      response.each_header { |name,value|
        puts "#{name}: #{value}"
      }
      puts "\n#{response.body}\n"
      puts "------------------------------------------------------------------"
    end
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Utility function to perform raw HTTP request.
  #
  # If url is not provided, will attempt to use generic 'server' and 'server_port' values from state.
  #
  def get_http_response(req, state, url)

    # TODO: SSL support

    auth = state.get('auth')
    if (auth == "basic")
      user = state.get('auth_user')
      password = state.get('auth_password')
      req.basic_auth(user, password)
    end

    if (url == nil)
      server = state.get('server')
      port = state.get('server_port')
      die("no server specified") if (server == nil)
      die("no server port specified") if (port == nil)
      url = "http://#{server}:#{port}"
    end

    begin
      urlobj = URI.parse(url)
      http = Net::HTTP.new(urlobj.host, urlobj.port)
      out(1, "Connecting to: #{urlobj.to_s}")
      response = http.request(req)

    rescue Exception => e
      return Result.fatal_error("Failure connecting to server #{server}: #{e.to_s}")
    end

    return response
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Utility function to perform HTTP request.
  # This is the method API subclasses should usually invoke when Discovery is available.
  # It will connect to a service of the given type.
  # It will log output and return a Result object.
  #
  def do_request_by_service(req, state, service)
    service_list = state.get('service_list')
    if (service_list == nil)
      return Result.fatal_error("service_list not available in state!")
    end

    if (service_list[service] == nil)
      return Result.fatal_error("No service of type #{service} available!")
    end

    url = service_list[service][0]
    return do_request_by_url(req, state, url)
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Utility function to perform HTTP request.
  # API subclasses can invoke this to connect to a specific host/port.
  # It will log output and return a Result object.
  #
  def do_request_by_url(req, state, url)
    show_req(req)
    response = get_http_response(req, state, url)
    show_response(response)

    state.set_in_test('http_code', response.code)
    result = Result.new(response)

    return result
  end

end
