#
# The State class is a container for both global and per-test state.
# The test framework keeps one global State object (in $GLOBAL_STATE).
# A per-test State object is created for each test. The per-test object
# inherits all the global state but can override values if needed.
# Values set in the per-test class are not persisted beyond the lifetime
# of a single test.
#
class State

  attr_accessor :vars

  #-------------------------------------------------------------------------------------------------------------------
  # Constructor. If baseobj is nil, a top-level State object is created. Otherwise, a child (per-test) State
  # object is created.
  #
  def initialize(baseobj = nil)
    @vars = Hash.new()
    @validations = Hash.new()

    @headers = Hash.new
    @headers['Content-Type'] = 'application/json'
    @headers['User-Agent'] = 'restest'

    @baseobj = baseobj
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Get a named value. Value is returned from either self, if present, or baseobj, if applicable.
  #
  def get(name)
    if (@vars[name] != nil)
      return @vars[name]
    end

    if (@baseobj != nil)
      return @baseobj.get(name)
    end

    return nil
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Set a persistent value. For per-test State objects, this means the value is set in the parent (global) State
  # object so it will persist beyond one test.
  #
  def set(name, value)
    if (@baseobj != nil)
      @baseobj.set(name, value)
    else
      @vars[name] = value
    end
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Remove a persistent value.
  #
  def unset(name)
    if (@baseobj != nil)
      @baseobj.unset(name)
    else
      @vars.delete(name)
    end
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Set a non-persistent value in a per-test State object. Calling this method on the global State is not allowed.
  #
  def set_in_test(name, value)
    if (@baseobj != nil)
      @vars[name] = value
    else
      die("cannot set_in_test for top-level State (name=#{name}, value=#{value})")
    end
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Return HTTP headers for request. A minimal default set of headers is returned.
  #
  # If a mix Hash is provided, its elements are merged to the default headers (headers in 'mix' override the defaults
  # if both are present). This allows per-test header customization.
  #
  def headers(mix = nil)
    if (mix != nil)
      return @headers.merge(mix)
    else
      return @headers
    end
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Record a per-test validation (loaded from test file). Calling this method on the global State is not allowed.
  #
  def set_validate(name, value)
    if (@baseobj != nil)
      @validations[name] =value
    else
      die("cannot set_validate for top-level State (name=#{name}, value=#{value})")
    end
  end

  #-------------------------------------------------------------------------------------------------------------------
  # Validates all the 'validate' conditions for the current test.
  # Calling this method on the global State is not allowed.
  # Validating verifies that all entries set with set_validate have a matching value in test state.
  # If any one or more fail, the result is marked as erroneous.
  #
  def validate(result)
    @validations.each_key { |key|
      failed = false
      val = get(key)
      wanted = @validations[key]
      wanted =~ /(\S+) (.*)/
      op = $1
      target = $2
      out(1, "validating: [#{key}]: want: [#{op} #{target}], got [#{val}]")
      if (op == "=")
        failed = true if (target != val.to_s)

      elsif (op == "<")
        failed = true if (val.to_i >= target.to_i)

      elsif (op == ">")
        failed = true if (val.to_i <= target.to_i)

      elsif (op == "is")
        if (target == "set")
          failed = true if (val == nil || val.length == 0)
        elsif (target == "unset")
          failed = true if (val != nil && val.length > 0)
        else
          die("unknown directive for operation 'is': #{target}")
        end

      else
        die("unknown validate operation #{op}")
      end

      if (failed)
        result.error("For [#{key}] expected [#{op} #{target}] but got [#{val}].")
        $VALIDATIONS_FAIL += 1
      else
        $VALIDATIONS_OK += 1
      end
    }

    # This is Proofpoint specific and should not be in this class. There's not a good place to put it right
    # now that doesn't involve copying multiple times, so put it here for now.
    if (result.code == "500")
      if (result.body != nil)
        begin
          json = JSON.parse(result.body)
          if (json != nil && json['host'] != nil && json['message'] != nil)
            result.add_message("Server: [#{json['host']}] said: [#{json['message']}])")
          end
        rescue
        end
      end
    end

  end

  #-------------------------------------------------------------------------------------------------------------------
  # To string...
  #
  def to_s
    if (@baseobj == nil)
      s = "Base State: "
      @vars.each_key { |key|
        s += "#{key}='#{@vars[key]}' "
      }
    else
      s = "Test State: "
      @vars.each_key { |key|
        s += "#{key}='#{@vars[key]}' "
      }
      s += "(" + @baseobj.to_s + ")"
    end
    return s
  end

end
